package com.phlx.wool.ui.slaughter;

import android.Manifest;
import android.app.Dialog;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.FrameLayout;

import androidx.databinding.Observable;
import androidx.lifecycle.ViewModelProviders;

import com.bigkoo.pickerview.builder.TimePickerBuilder;
import com.bigkoo.pickerview.listener.OnTimeSelectChangeListener;
import com.bigkoo.pickerview.listener.OnTimeSelectListener;
import com.bigkoo.pickerview.view.TimePickerView;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.request.RequestOptions;
import com.phlx.wool.BR;
import com.phlx.wool.R;
import com.phlx.wool.data.VMFactory;
import com.phlx.wool.databinding.ActivitySlaughterEntryBinding;
import com.phlx.wool.entity.Label;
import com.phlx.wool.ui.base.BaseActivity;
import com.phlx.wool.ui.slaughter.vm.SlaughterEntryVM;
import com.phlx.wool.ui.villus.vm.VillusBuyVM;
import com.phlx.wool.util.BaseImageUtils;
import com.phlx.wool.util.ImagePicker;
import com.tbruyelle.rxpermissions2.RxPermissions;

import java.util.ArrayList;
import java.util.Date;

import io.reactivex.functions.Consumer;
import me.goldze.mvvmhabit.base.BaseViewModel;
import me.goldze.mvvmhabit.utils.ToastUtils;

public class SlaughterEntryActivity extends BaseActivity<ActivitySlaughterEntryBinding, SlaughterEntryVM> {

    private ArrayList<Label> labels;

    private ImagePicker quarantineImagePicker;

    private TimePickerView pvTime;

    @Override
    public int initContentView(Bundle savedInstanceState) {
        return R.layout.activity_slaughter_entry;
    }

    @Override
    public int initVariableId() {
        return BR.viewModel;
    }

    @Override
    public void initParam() {
        super.initParam();
        Bundle mBundle = getIntent().getExtras();
        if (mBundle != null) {
            labels = mBundle.getParcelableArrayList("list");
        }
    }

    @Override
    public SlaughterEntryVM initViewModel() {
        VMFactory factory = VMFactory.getInstance(getApplication());
        return ViewModelProviders.of(this, factory).get(SlaughterEntryVM.class);
    }

    @Override
    public void initData() {
        super.initData();

        viewModel.setLabels(labels);
        viewModel.initToolbar();

        initTimePicker();

        if (quarantineImagePicker == null) {
            quarantineImagePicker = new ImagePicker(SlaughterEntryActivity.this);
            quarantineImagePicker.setOnImagePickerListener(new ImagePicker.OnImagePickerListener() {
                @Override
                public void onSuccess(String path) {
                    Bitmap bitmap = BaseImageUtils.decodeBitmap(path, SlaughterEntryActivity.this);
                    BaseImageUtils.saveBitmapToSDCard(bitmap, viewModel.getQuarantineImgUrl());
                    viewModel.entity.setQuarantineImgPath(viewModel.getQuarantineImgUrl());
                    Glide.with(SlaughterEntryActivity.this)
                            .load(viewModel.getQuarantineImgUrl())
                            .apply(
                                    new RequestOptions()
                                            .skipMemoryCache(true)
                                            .diskCacheStrategy(DiskCacheStrategy.NONE)
                                            .error(R.drawable.no_image)
                                            .fallback(R.drawable.no_image)
                            )
                            .into(binding.quarantinePhotoView);
                }
            });
        }

    }

    @Override
    public void initViewObservable() {
        super.initViewObservable();

        viewModel.entryDay.observe(this, aBoolean -> {
            pvTime.show(binding.entryDayTv);
        });

        viewModel.quarantinePhotoOnClick.observe(this, aBoolean -> {
                    RxPermissions rxPermissions = new RxPermissions(SlaughterEntryActivity.this);
                    rxPermissions.request(Manifest.permission.CAMERA, Manifest.permission
                            .READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE)
                            .subscribe(new Consumer<Boolean>() {
                                @Override
                                public void accept(Boolean aBoolean) throws Exception {
                                    if (aBoolean) {
                                        quarantineImagePicker.show(SlaughterEntryActivity.this);
                                    } else {
                                        ToastUtils.showShort("权限被拒绝");
                                    }
                                }
                            });
                });
    }

    private void initTimePicker() {//Dialog 模式下，在底部弹出
        pvTime = new TimePickerBuilder(this, new OnTimeSelectListener() {
            @Override
            public void onTimeSelect(Date date, View v) {
                viewModel.setEntryDate(date);
            }
        })
                .setTimeSelectChangeListener(new OnTimeSelectChangeListener() {
                    @Override
                    public void onTimeSelectChanged(Date date) {
                    }
                })
                .setType(new boolean[]{true, true, true, false, false, false})
                .isDialog(true) //默认设置false ，内部实现将DecorView 作为它的父控件。
                .addOnCancelClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        Log.i("pvTime", "onCancelClickListener");
                    }
                })
                .setItemVisibleCount(5) //若设置偶数，实际值会加1（比如设置6，则最大可见条目为7）
                .setLineSpacingMultiplier(2.0f)
                .isAlphaGradient(true)
                .build();

        Dialog mDialog = pvTime.getDialog();
        if (mDialog != null) {

            FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT,
                    Gravity.BOTTOM);

            params.leftMargin = 0;
            params.rightMargin = 0;
            pvTime.getDialogContainerLayout().setLayoutParams(params);

            Window dialogWindow = mDialog.getWindow();
            if (dialogWindow != null) {
                dialogWindow.setWindowAnimations(com.bigkoo.pickerview.R.style.picker_view_slide_anim);//修改动画样式
                dialogWindow.setGravity(Gravity.BOTTOM);//改成Bottom,底部显示
                dialogWindow.setDimAmount(0.3f);
            }
        }
    }
}
